"use client";
import { useState } from "react";
import { Button, Modal } from "react-bootstrap";
import { useFormState } from "react-dom";

import { changePasswordAction, ChangePasswordState } from "./changePwAction";

import { FormError } from "components/form/alert";
import { SubmitButton } from "components/form/buttons";
import { PasswordInput } from "components/form/inputs/text";
import { Item } from "components/form/items";
import { validatePW } from "components/form/validation";

/* -------------------------------------------------------------------------- */
/*                               Password Change                              */
/* -------------------------------------------------------------------------- */

export function ChangePassword() {
    const [show, setShow] = useState(false);
    const handleClose = () => setShow(false);
    const handleShow = () => setShow(true);
    return (
        <>
            <Button variant="outline-dark" onClick={handleShow}>
                Change password
            </Button>
            <Modal
                show={show}
                onHide={handleClose}
                centered
                size="lg"
                fullscreen="md-down"
                backdrop="static"
                keyboard={false}
            >
                <Modal.Header closeButton>
                    <Modal.Title>Change your Password</Modal.Title>
                </Modal.Header>
                <Modal.Body>
                    <Item
                        title="New password"
                        description="Please enter your new password and confirm it."
                    >
                        <ChangePasswordForm />
                    </Item>
                </Modal.Body>
            </Modal>
        </>
    );
}

function ChangePasswordForm() {
    const [state, formAction] = useFormState(changePassword, {
        currentPassword: -1,
        newPassword: -1,
        confirmNewPassword: -1,
    });

    return (
        <form action={formAction} className="mt-3 d-flex gap-2 flex-column">
            <FormError>{state.message}</FormError>
            <PasswordInput
                name="currentPassword"
                label="Current password"
                isValid={state.currentPassword == 1}
                isInvalid={state.currentPassword == 0}
                disabled={state.passwordChanged}
                invalidFeedback="Invalid or incorrect password!"
            />
            <PasswordInput
                name="newPassword"
                label="New password"
                isValid={state.newPassword == 1}
                isInvalid={state.newPassword == 0}
                disabled={state.passwordChanged}
            />
            <PasswordInput
                name="confirmNewPassword"
                label="Confirm new password"
                invalidFeedback="Passwords do not match!"
                isValid={state.confirmNewPassword == 1}
                isInvalid={state.confirmNewPassword == 0}
                disabled={state.passwordChanged}
            />
            <SubmitButton disabled={state.passwordChanged}>
                {state.passwordChanged
                    ? "Password changed!"
                    : "Change password"}
            </SubmitButton>
        </form>
    );
}

async function changePassword(
    _previousState: ChangePasswordState,
    formData: FormData,
): Promise<ChangePasswordState> {
    const ret = {
        currentPassword: -1,
        newPassword: -1,
        confirmNewPassword: -1,
    } as ChangePasswordState;

    if (formData.has("currentPassword")) {
        const currentPassword: string = formData.get(
            "currentPassword",
        )! as string;
        ret.currentPassword = validatePW(currentPassword) ? 1 : 0;
    }
    if (formData.has("newPassword")) {
        const newPassword: string = formData.get("newPassword")! as string;
        ret.newPassword = validatePW(newPassword) ? 1 : 0;
        if (formData.has("confirmNewPassword")) {
            ret.confirmNewPassword =
                formData.get("newPassword") ===
                formData.get("confirmNewPassword")
                    ? 1
                    : 0;
        }
    }

    if (
        ret.currentPassword != 1 ||
        ret.newPassword != 1 ||
        ret.confirmNewPassword != 1
    ) {
        return ret;
    }
    return await changePasswordAction(ret, formData);
}
