import type { Config } from "jest";
import nextJest from "next/jest.js";

const createJestConfig = nextJest({
    // Provide the path to your Next.js app to load next.config.js and .env files in your test environment
    dir: "./",
});

// Add any custom config to be passed to Jest
var config: Config = {
    displayName: "fullstack api tests",
    preset: "ts-jest",

    testEnvironment: "node",
    setupFilesAfterEnv: ["<rootDir>/__mocks__/global_mocks.ts"],
    moduleDirectories: ["node_modules", "<rootDir>"],
    moduleNameMapper: {
        "^lib/(.*)$": "<rootDir>/lib/$1",
        "^.+\\.(css|style|less|sass|scss|png|jpg|ttf|woff|woff2|svg)$":
            "jest-transform-stub",
    },
    // Collect coverage from api and lib
    collectCoverage: true,
    coverageProvider: "v8",
    coverageReporters: ["json", "lcov", "text", "clover"],
    collectCoverageFrom: [
        "app/api/**/*.{ts,tsx}",
        "!**/*.d.ts",
        "!**/node_modules/**",
        "!**/vendor/**",
        "!**/.next/**",
        //Any config files
        "!**/jest.config.ts",
    ],
};

// createJestConfig is exported this way to ensure that next/jest can load the Next.js config which is async
module.exports = createJestConfig(config);
